#define use_display 1

IntervalTimer myTimer;
float tempo = 140.0;

#ifdef use_display

/* -------------------------------------------------------------------------------
    Everything needed for the display
   ---------------------------------------------------------------------------- */
#include <TM1637Display.h>

// Lines for TM1637
#define clk  0
#define data 1

TM1637Display display(clk, data);
bool isOn = false;

void setDisplay(float tmp)
{
  display.showNumberDecEx((int)tmp, (0x80 >> 2), true);
}

void flash_clock() {
  if(isOn)
    display.setBrightness(0x0, true);
  else
    display.setBrightness(0x7, true);
  isOn = !isOn;
  setDisplay(tempo * 10);
}

void install_flash_timer()
{
  #ifdef use_display
    myTimer.end();
    myTimer.begin (flash_clock, 500000);
  #endif
}

void setDisplayBrightness(int brightness)
{
  display.setBrightness(brightness, true);
}
#else
/* -------------------------------------------------------------------------------
    Dummy functions when use_display is not defined
   ---------------------------------------------------------------------------- */
void setDisplay(float tmp){}
void install_flash_timer(){}
void setDisplayBrightness(int brightness){}
#endif
//-------------------------------------------------------------------------------

volatile uint32_t iv = 0;
 // SysEx array. Using a big array and sending it seems to help flush the usb.
 // Not too sure about it though
uint8_t b[512];
int start = 0;
float lsbSpeed = -1;
float msbSpeed = -1;

void send_clock() {
  b[0] = 0xf8;

  for (int x = 0; x < 16; x++) {
    b[0] = 0xf8;
    // Might be more stable...
    usbMIDI.sendSysEx(512, b, true, x);
  }
  usbMIDI.send_now();
}

/*
  Controller legenda:
  CC 17, value 1       Toggle clock on/off
  CC 17, value 2       Switch clock on
  CC 17, value 3       Switch clock off
  CC 17, value 16      Increase clock speed by 1
  CC 17, value 17      Decrease clock speed by 1
  CC 17, value 128     Get current state
  CC 16                Sets msb of absolute speed (in tenths!)
  CC 48                Sets lsb of absolute speed (in tenths!)

  To set the absolute speed to 143.7 msb and lsb together send
  the value 1437:
 
    msb:  1437 / 128 = 11
    lsb:  1437 % 128 = 29

  Check:

    (11 * 128) + 29 = 1437
 
  Midi to send: 

    CC16, value 11
    CC48, value 29
*/
void myControlChange(byte channel, byte control, byte value) {
  if (control == 17) { // On/Off, Toggle on/off, Speed increase, Speed decrease,
    if (value == 1) {           // Toggle on/off
      if (start == 1) {
        install_flash_timer();
        for (int x = 0; x < 16; x++)
          usbMIDI.sendRealTime(usbMIDI.Stop, x);
        usbMIDI.send_now();
        start = 0;
      } else {
        for (int x = 0; x < 16; x++)
          usbMIDI.sendRealTime(usbMIDI.Start, x);
        usbMIDI.send_now();
        myTimer.end();
        myTimer.begin(send_clock, iv);
        start = 1;
      }
    } else if (value == 2) {    // Switch On
      if (start == 0) {
        for (int x = 0; x < 16; x++)
          usbMIDI.sendRealTime(usbMIDI.Start, x);
        usbMIDI.send_now();
        myTimer.end();
        myTimer.begin(send_clock, iv);
        start = 1;
      }
    } else if (value == 3) {    // Switch Off
      if (start == 1) {
        myTimer.end();
        for (int x = 0; x < 16; x++)
          usbMIDI.sendRealTime(usbMIDI.Stop, x);
        
        install_flash_timer();
        usbMIDI.send_now();
        start = 0;
      }
    } else if (value == 16) {   // Increase tempo
      tempo++;
      iv = ((60000000.0 / (tempo)) / 24.0);
      if (start)
        myTimer.update(iv);
    } else if (value == 17) {   // Decrease tempo
      tempo--;
      iv = ((60000000.0 / (tempo)) / 24.0);
      if (start)
        myTimer.update(iv);
    } else if (value == 127) {  // Request for current state
      b[0] = 0xf0;
      b[1] = 0x7f;
      b[2] = 0x7f;
      b[3] = start;                               // Clock is running?
      b[4] = (((int)(tempo * 10)) / 128) & 0x7f;  // Speed msb
      b[5] = (((int)(tempo * 10)) % 128) & 0x7f;  // Speed lsb
      b[6] = 0xf7;
      for (int x = 0; x < 16; x++) {
        usbMIDI.sendSysEx(7, b, true, x);
      }

      for (int x = 0; x < 7; x++)
        b[x]=0;
    }
  } else {
    if (control == 16) { // MSB Absolute speed in tenths
      msbSpeed = value;
    }

    if (control == 48) { // LSB Absolute speed in tenths
      lsbSpeed = value;
    }

    // Only when msb and lsb has been send, the speed is adjusted 
    if (msbSpeed != -1 && lsbSpeed != -1) {
      tempo = ((msbSpeed * 128) + lsbSpeed) / 10.0;
      msbSpeed = -1;
      lsbSpeed = -1;
      iv = ((60000000.0 / (tempo)) / 24.0);
      if(start)
        myTimer.update(iv);
    }
  }


  if (start == 1)
    setDisplayBrightness(0x7);
  setDisplay(tempo * 10);
}



void setup() {
  iv = ((60000000.0 / tempo) / 24.0);
  b[0] = 0xf8;

  usbMIDI.setHandleControlChange(myControlChange);
  myTimer.priority(0);
  #ifdef use_display
  display.setBrightness(0x1, true);
  setDisplay(tempo * 10);
  myTimer.begin (flash_clock, 500000);
  #endif
}

void loop() {
  while (usbMIDI.read()) {}
  //delay(1000);
}
